/*-----------------------------------------------------------------------------
  name:         lcd_gps_clock
  description:  LCD/GPS sample clockwith time zone knob
  Version:      2
  By:           Tim Mikkelsen
  Created:      3/22/2016
  Updated:      4/29/2016

  This Arduino code/project is for an LCD clock using
  a NEMA style GPS and 2 line LCD display

  Hardware:
  - Arduino - Arduino Pro Mini 8Mhz/3.3V      (Sparkfun DEV-11114)
  - Basic 16x2 LCD display - HD44780 chipset  (Sparkfun LCD-00791)
  - GPS Receiver - GP-20U7                    (Sparkfun GPS-13470)
  - two 10k trimpots                          (Sparkfun COM-09806)
  - FTDI Basic Breakout (to program Arduino)  (Sparkfun DEV-09873)
  - 6v wall power supply

  Additional libraries:
  - TinyGPS++ (by Mikal Hart)
  - timezone adjustment code (by Jinseok Jeon)

  Note the use of smartDelay to ensure not losing data - from the
  TinyGPS++ example.  There are still occasional checksum errors
  and overflows, but the data does get read
*/

// libraries  -----------------------------------------------------------------
#include <TinyGPS++.h>
#include <SoftwareSerial.h>
#include <LiquidCrystal.h>
#include <string.h>
#include <ctype.h>

// constants  -----------------------------------------------------------------
static const int TZPotPin = 0;            // timezone pot pin defintion     A0
static const int WorkingPin = 13;         // gps working pin definition     D13
static const int RXPin = 8;               // gps serial Rx pin definition   D8
static const int TXPin = 9;               // gps serial Tx pin definition   D9
static const uint32_t GPSBaud = 9600;     // gps baud rate
// static const uint32_t DbgBaud = 9600;  // debug serial monitor baud rate

// variables  -----------------------------------------------------------------
int gpsYear;
int gpsMonth;
int gpsDay;
int gpsHour;
int TZPotValue;                           // Timezone pot input on A0 input
int TimeZone;                             // Timezone
byte gpsMin;
byte gpsSec;
String TimeStr;
String DateStr;
String FirstName = "Gio";                 // <= 7 characters for first name
String LastName = "Falbo";                // <= 7 characters for last name
int DSTbegin[] = {                        // DST 2015 - 2032 in Canada and US
  308, 313, 312, 311, 310, 308, 314, 313, 312,            // (MMdd coding)
  310, 309, 308, 314, 312, 311, 310, 309, 314
};
int DSTend[] = {                          // DST 2015 - 2032 in Canada and US
  1101, 1106, 1105, 1104, 1103, 1101, 1107, 1106, 1105,   // (MMdd coding)
  1103, 1102, 1101, 1107, 1105, 1104, 1103, 1102, 1107
};
int DaysAMonth[] = {                      //number of days a month
  31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
};


// create devices  ------------------------------------------------------------
TinyGPSPlus gps;                          // TinyGPS++ object
SoftwareSerial ss(RXPin, TXPin);          // serial connection to the GPS device
LiquidCrystal lcd(12, 11, 5, 4, 3, 2);    // LCD device/pin-out definition


// This custom version of delay() ensures that the gps object
// is being "fed".
static void smartDelay(unsigned long ms)
{
  unsigned long start = millis();
  do
  {
    while (ss.available())
      gps.encode(ss.read());
  } while (millis() - start < ms);
}


// set up SmartSerial
void setup()
{
  // set up GPS serial interface  ---------------------------------------------
  ss.begin(GPSBaud);

  // set up LCD  --------------------------------------------------------------
  lcd.begin(20, 4);
  lcd.print("waiting for gps");
}


// main loop of program
void loop()
{
  // display information every time a new stream is correctly read  -----------
  while (ss.available() > 0)
    if (gps.encode(ss.read()))
      displayInfo();
  smartDelay(0);
}

void displayInfo()
{

  // make sure gps data is good  ----------------------------------------------
  if (gps.date.day() == 0 &&
      gps.date.month() == 0)
  {
    lcd.clear();
    lcd.print(F("acquiring gps"));
    digitalWrite(WorkingPin, LOW);
  }
  else {

    // clear out the time and date strings
    TimeStr = "";
    DateStr = "";

    // turn on LED on pint 13 to show functioning
    digitalWrite(WorkingPin, HIGH);

    // get actual UTC gps data
    gpsYear = gps.date.year();
    gpsMonth = gps.date.month();
    gpsDay = gps.date.day();
    gpsHour = gps.time.hour();

    // get timezone input -----------------------------------------------------
    TZPotValue = analogRead(TZPotPin);
    TimeZone = ((TZPotValue - 512) / 40);
    if (TimeZone < -12) TimeZone = -12;
    if (TimeZone > 12) TimeZone = 12;

    // leap year check  -------------------------------------------------------
    if (gps.date.year() % 4 == 0) DaysAMonth[1] = 29;
    else DaysAMonth[1] = 28;

    // time zone adjustment  --------------------------------------------------
    gpsHour += TimeZone;

    // daylight savings time adjustments  -------------------------------------
    if (gpsMonth * 100 + gpsDay >= DSTbegin[gpsYear - 13] &&
        gpsMonth * 100 + gpsDay < DSTend[gpsYear - 13]) gpsHour += 1;
    if (gpsHour < 0)
    {
      gpsHour += 24;
      gpsDay -= 1;
      if (gpsDay < 1)
      {
        if (gpsMonth == 1)
        {
          gpsMonth = 12;
          gpsYear -= 1;
        }
        else
        {
          gpsMonth -= 1;
        }
        gpsDay = DaysAMonth[gpsMonth - 1];
      }
    }
    if (gpsHour >= 24)
    {
      gpsHour -= 24;
      gpsDay += 1;
      if (gpsDay > DaysAMonth[gpsMonth - 1])
      {
        gpsDay = 1;
        gpsMonth += 1;
        if (gpsMonth > 12) gpsYear += 1;
      }
    }

    lcd.setCursor(0, 0);    // position on the first line of the display

    smartDelay(0);

    // date checks  -----------------------------------------------------------
    // note there are only 16 characters for the message     1234567890123456
    if      ((gpsMonth == 1)  && (gpsDay == 1))   lcd.print("Happy New Year  ");
    else if ((gpsMonth == 2)  && (gpsDay == 10))  lcd.print("Bishop's b-day  ");
    else if ((gpsMonth == 2)  && (gpsDay == 12))  lcd.print("Matilda's b-day ");
    else if ((gpsMonth == 3)  && (gpsDay == 4))   lcd.print("Gigi's birthday ");
    else if ((gpsMonth == 4)  && (gpsDay == 22))  lcd.print("Happy Earth Day ");
    else if ((gpsMonth == 5)  && (gpsDay == 1))   lcd.print("Gio's birthday  ");
    else if ((gpsMonth == 6)  && (gpsDay == 14))  lcd.print("Flag Day        ");
    else if ((gpsMonth == 7)  && (gpsDay == 4))   lcd.print("Independence Day");
    else if ((gpsMonth == 7)  && (gpsDay == 27))  lcd.print("Bopa's birthday ");
    else if ((gpsMonth == 9)  && (gpsDay == 9))   lcd.print("Dad's birthday  ");
    else if ((gpsMonth == 12) && (gpsDay == 25))  lcd.print("Merry Christmas ");
    else if ((gpsMonth == 12) && (gpsDay == 27))  lcd.print("Mom's birthday  ");
    else {

      smartDelay(0);

      // build the date string ------------------------------------------------

      if (gpsMonth < 10) DateStr = "0";
      DateStr += gpsMonth;
      DateStr += "/";
      if (gpsDay < 10) DateStr += "0";
      DateStr += gpsDay;
      DateStr += "/";
      gpsYear = gpsYear % 100;
      if (gpsYear < 10) DateStr += "0";
      DateStr += gpsYear;

      // have enough blanks for the first name
      for (int i = 0; i < ( 8 - FirstName.length()) ; i++)
        DateStr = (" ") + DateStr;

      lcd.print(FirstName + DateStr);   // display the first name and the date
    }

    smartDelay(0);

    lcd.setCursor(0, 1);    // position on the second line of the display

    // build the date string --------------------------------------------------

    if (gpsHour < 10) TimeStr = "0";
    TimeStr += gpsHour;
    TimeStr += ":";
    if (gps.time.minute() < 10) TimeStr += "0";
    TimeStr += gps.time.minute();
    TimeStr += ":";
    if (gps.time.second() < 10) TimeStr += "0";
    TimeStr += gps.time.second();

    // have enough blanks for the last name
    for (int i = 0; i < ( 8 - LastName.length()) ; i++)
      TimeStr = (" ") + TimeStr;

    lcd.print(LastName + TimeStr);   // display the last name and the time

  }

  smartDelay(0);

}











